package Game
{
	import starling.core.Starling;
	import starling.display.Image;
	import starling.display.Quad;
	import starling.display.QuadBatch;
	import starling.display.Sprite;
	import starling.filters.BlurFilter;
	
	public class Background extends Sprite
	{
		public var game:GameScene;
		public var quads:Array;
		public var batch:QuadBatch;
		public var helperQuad:Quad;
		public var whiteQuad:Quad;
		public var redQuad:Quad;
		
		public function Background(game:GameScene)
		{
			super();
			this.game = game;
			addChild(new Image(Root.assets.getTexture("bg")));
			
			quads = new Array();
			for(var i:int = 0; i < 300; i++) {
				var quad:BGQuad = new BGQuad(Math.random()*900-50, Math.random()*900-50, Math.random()*0.75+0.25, Math.random()*0.5+0.5);
				quads.push(quad);
			}
			
			helperQuad = new Quad(10, 10, 0xff00ff);
			helperQuad.rotation = Math.PI*0.25;
			helperQuad.blendMode = "add";
			batch = new QuadBatch();
			addChild(batch);
			
			whiteQuad = new Quad(800, 600, 0xffffff);
			whiteQuad.blendMode = "add";
			whiteQuad.alpha = 0;
			addChild(whiteQuad);
			
			redQuad = new Quad(800, 600, 0xff0000);
			redQuad.blendMode = "add";
			redQuad.alpha = 0;
			addChild(redQuad);
			
			//this.filter = new BlurFilter(1, 1, 1);
		}
		
		public function init():void {
			whiteQuad.alpha = 0;
			redQuad.alpha = 0;
		}
		
		public function deInit():void {
			
		}
		
		public function update(dt:Number, scrollX:Number, scrollY:Number):void {
			batch.reset();
			
			var l:int = quads.length;
			for(var i:int = 0; i < l; i++) {
				var quad:BGQuad = quads[i];
				quad.x += scrollX*quad.scale;
				quad.y += scrollY*quad.scale;
				
				if(quad.x < -50) quad.x = 900;
				if(quad.x > 900) quad.x = -50;
				if(quad.y < -50) quad.y = 900;
				if(quad.y > 900) quad.y = -50;
				
				helperQuad.x = quad.x;
				helperQuad.y = quad.y;
				helperQuad.scaleX = helperQuad.scaleY = quad.scale;
				helperQuad.alpha = 0.2* quad.alpha;
				
				batch.addQuad(helperQuad);
			}
		}
		
		public function highlight():void {
			Starling.juggler.removeTweens(whiteQuad);
			Starling.juggler.tween(whiteQuad, 0.05, {alpha:0.1});
			Starling.juggler.tween(whiteQuad, 0.2, {alpha:0, delay:0.05});
		}
		
		public function showHit():void {
			Starling.juggler.removeTweens(redQuad);
			Starling.juggler.tween(redQuad, 0.05, {alpha:0.3});
			Starling.juggler.tween(redQuad, 0.2, {alpha:0, delay:0.05});
		}
	}
}