package Game
{
	import starling.display.Quad;
	import starling.display.QuadBatch;
	import starling.filters.BlurFilter;

	public class GameFX
	{
		public var game:GameScene;
		public var trailBatch:QuadBatch;
		public var trailQuad:Quad;
		public var bulletQuad:Quad;
		public var arrowQuad:Quad;
		public var powerQuad:Quad;
		public var pulseQuad:Quad;
		
		public var bf:BlurFilter;
		
		public function GameFX(game:GameScene)
		{
			this.game = game;
			
			trailBatch = new QuadBatch();
			trailQuad = new Quad(10, 4, 0x85faff);
			trailQuad.pivotX = 0;
			trailQuad.pivotY = 2;
			trailQuad.blendMode = "add";
			
			bulletQuad = new Quad(30, 6, 0xffffff);
			bulletQuad.mVertexData.setPosition(0, -20, 0);
			bulletQuad.mVertexData.setPosition(1, 0, -4);
			bulletQuad.mVertexData.setPosition(2, 0, 4);
			bulletQuad.mVertexData.setPosition(3, 20, 0);
			bulletQuad.mVertexData.setAlpha(0, 0);
			bulletQuad.mVertexData.setAlpha(1, 0.5);
			bulletQuad.mVertexData.setAlpha(2, 0.5);
			bulletQuad.blendMode = "add";
			
			arrowQuad = new Quad(10, 10, 0xff0000);
			arrowQuad.mVertexData.setPosition(0, 0, -12);
			arrowQuad.mVertexData.setPosition(1, 24, 0);
			arrowQuad.mVertexData.setPosition(2, 0, 12);
			arrowQuad.mVertexData.setPosition(3, 24, 0);
			arrowQuad.blendMode = "add";
			
			powerQuad = new Quad(20, 20, 0x00aaff);
			powerQuad.mVertexData.setPosition(0, 0, -20);
			powerQuad.mVertexData.setPosition(1, -20, 0);
			powerQuad.mVertexData.setPosition(2, 20, 0);
			powerQuad.mVertexData.setPosition(3, 0, 20);
			powerQuad.blendMode = "add";
			
			pulseQuad = new Quad(20, 20, 0xffffff);
			pulseQuad.mVertexData.setPosition(0, 0, -20);
			pulseQuad.mVertexData.setPosition(1, -20, 0);
			pulseQuad.mVertexData.setPosition(2, 20, 0);
			pulseQuad.mVertexData.setPosition(3, 0, 20);
			pulseQuad.blendMode = "add";
			
			bf =  new BlurFilter(1, 1, 1);
		}
		
		public function init():void {
			game.trailLayer.addChild(trailBatch);
		}
		
		public function deInit():void {
			game.trailLayer.removeChild(trailBatch);
		}
		
		public function update(dt:Number):void {
			trailBatch.reset();
			
			var l:int = game.trails.length;
			for(var i:int = 0; i < l; i++) {
				var trail:TrailFX = game.trails[i];
				trailQuad.x = trail.x;
				trailQuad.y = trail.y;
				trailQuad.rotation = 0;
				trailQuad.width = trail.width;
				trailQuad.rotation = trail.rotation;
				trailQuad.alpha = trail.alpha;
				trailQuad.color = trail.color;
				trailBatch.addQuad(trailQuad);
			}
			
			l = game.projectiles.length;
			for(i = 0; i < l; i++) {
				var bullet:Projectile = game.projectiles[i];
				bulletQuad.x = bullet.image.x;
				bulletQuad.y = bullet.image.y;
				bulletQuad.rotation = bullet.image.rotation;
				bulletQuad.alpha = bullet.image.alpha;
				bulletQuad.color = bullet.image.color;
				trailBatch.addQuad(bulletQuad);
			}
			
			arrowQuad.color = 0xff0000;
			l = game.ships.length;
			for(i = 0; i < l; i++) {
				var ship:Ship = game.ships[i];
				var dx:Number = ship.x - game.hero.x;
				var dy:Number = ship.y - game.hero.y;
				var dist:Number = Math.sqrt(dx*dx + dy*dy);
				if(dist > 800) {
					var angle:Number = Math.atan2(dy, dx);
					arrowQuad.rotation = angle;
					arrowQuad.x = -game.mainLayer.x*2 + 800 + (game.ui.hud.x*2) + Math.cos(angle)*500;
					arrowQuad.y = -game.mainLayer.y*2 + 600 + (game.ui.hud.y*2) + Math.sin(angle)*500;
					trailBatch.addQuad(arrowQuad);
				}
			}
			
			arrowQuad.color = 0x00aaff;
			l = game.powerups.length;
			for(i = 0; i < l; i++) {
				var powerup:PowerUp = game.powerups[i];
				powerQuad.x = pulseQuad.x = powerup.x;
				powerQuad.y = pulseQuad.y = powerup.y;
				powerQuad.scaleX = powerQuad.scaleY = powerup.scale;
				powerQuad.alpha = powerup.alpha;
				trailBatch.addQuad(powerQuad);
				
				pulseQuad.alpha = powerup.pulse * powerup.alpha;
				pulseQuad.scaleX = pulseQuad.scaleY = powerup.scale;
				trailBatch.addQuad(pulseQuad);
				
				dx = powerup.x - game.hero.x;
				dy = powerup.y - game.hero.y;
				dist = Math.sqrt(dx*dx + dy*dy);
				if(dist > 800) {
					angle = Math.atan2(dy, dx);
					arrowQuad.rotation = angle;
					arrowQuad.x = -game.mainLayer.x*2 + 800 + (game.ui.hud.x*2) + Math.cos(angle)*500;
					arrowQuad.y = -game.mainLayer.y*2 + 600 + (game.ui.hud.y*2) + Math.sin(angle)*500;
					trailBatch.addQuad(arrowQuad);
				}
			}
		}
	}
}