package Game
{
	import flash.geom.Point;
	
	import starling.core.Starling;
	import starling.display.Image;
	import starling.display.Quad;
	import starling.display.Sprite;
	import starling.events.Touch;
	import starling.events.TouchEvent;
	import starling.events.TouchPhase;
	import starling.text.BitmapFont;
	import starling.text.TextField;

	public class GameUI extends Sprite
	{
		public var game:GameScene;
		public var hitQuad:Quad;
		public var mouseLoc:Point;
		public var mousePresent:Boolean;
		public var mouseDown:Boolean;
		
		public var hud:Image;
		public var timer1:TextField;
		public var timer2:TextField;
		public var quad1:Quad;
		public var quad2:Quad;
		public var quad3:Quad;
		public var quad4:Quad;
		
		public var cursor:Image;
		
		public function GameUI(game:GameScene)
		{
			this.game = game;
			
			hitQuad = new Quad(1600, 1000, 0x0000ff);
			hitQuad.x = -400;
			hitQuad.y = -300;
			hitQuad.alpha = 0;
			addChild(hitQuad);
			
			mouseLoc = new Point(0, 0);
			hud = new Image(Root.assets.getTexture("hud"));
			hud.color = 0x330033;
			hud.blendMode = "add";
			addChild(hud);
			
			timer1 = new TextField(100, 50, "00:00", "DS-Digital-Bold", BitmapFont.NATIVE_SIZE, 0x00aaff, true);
			timer1.hAlign = "left";
			timer1.vAlign = "top";
			timer1.blendMode = "add";
			timer1.x = -2;
			timer1.y = 340;
			timer1.rotation = -Math.PI*0.5;
			addChild(timer1);
			
			timer2 = new TextField(100, 50, "00:00", "DS-Digital-Bold", BitmapFont.NATIVE_SIZE, 0x00aaff, true);
			timer2.hAlign = "left";
			timer2.vAlign = "top";
			timer2.blendMode = "add";
			timer2.x = 802;
			timer2.y = 260;
			timer2.rotation = Math.PI*0.5;
			addChild(timer2);
			
			quad1 = new Quad(250, 10, 0x00aaff);
			quad1.blendMode = "add";
			quad1.rotation = -Math.PI*0.5;
			quad1.x = 10;
			quad1.y = 250;
			addChild(quad1);
			
			quad2 = new Quad(250, 10, 0x00aaff);
			quad2.blendMode = "add";
			quad2.rotation = Math.PI*0.5;
			quad2.x = 20;
			quad2.y = 350;
			addChild(quad2);
			
			quad3 = new Quad(250, 10, 0x00aaff);
			quad3.blendMode = "add";
			quad3.rotation = -Math.PI*0.5;
			quad3.x = 780;
			quad3.y = 250;
			addChild(quad3);
			
			quad4 = new Quad(250, 10, 0x00aaff);
			quad4.blendMode = "add";
			quad4.rotation = Math.PI*0.5;
			quad4.x = 790;
			quad4.y = 350;
			addChild(quad4);
			
			cursor = new Image(Root.assets.getTexture("halo"));
			cursor.pivotX = cursor.width*0.5;
			cursor.pivotY = cursor.height*0.5;
			cursor.scaleX = cursor.scaleY = 0.5;
			cursor.color = 0x330033;
			cursor.blendMode = "add";
			addChild(cursor);
		}
		
		public function init():void {
			timer1.text = "00:00";
			timer2.text = "00:00";
			quad1.scaleX = quad2.scaleX = quad3.scaleX = quad4.scaleX = 1;
			mouseLoc.x = 400;
			mouseLoc.y = 0;
			mousePresent = false;
			mouseDown = false;
			game.addChild(this);
			addEventListener(TouchEvent.TOUCH, on_touch);
		}
		
		public function reset():void {
			timer1.text = "00:00";
			timer2.text = "00:00";
			quad1.scaleX = quad2.scaleX = quad3.scaleX = quad4.scaleX = 1;
		}
		
		public function start():void {
			
		}
		
		public function deInit():void {
			game.removeChild(this);
			removeEventListener(TouchEvent.TOUCH, on_touch);
		}
		
		public function on_touch(e:TouchEvent):void {
			var touch:Touch = e.getTouch(this);
			if(!touch) {
				mousePresent = false;
				return;
			}
			
			mousePresent = true;
			switch(touch.phase) {
				case TouchPhase.HOVER:
					touch.getLocation(this, mouseLoc);
					break;
				case TouchPhase.BEGAN:
					mouseDown = true;
					touch.getLocation(this, mouseLoc);
					break;
				case TouchPhase.MOVED:
					touch.getLocation(this, mouseLoc);
					break;
				case TouchPhase.ENDED:
					if(game.gameOver) {
						if(game.results.alpha == 1) game.reset();
						else if(game.results.alpha == 0 && game.logo.alpha == 1) game.start();
					}
					mouseDown = false;
					break;
			}
		}
		
		public function update(dt:Number):void {
			if(mousePresent) {
				cursor.visible = true;
				cursor.x = mouseLoc.x;
				cursor.y = mouseLoc.y;
			} else {
				cursor.visible = false;
			}
			
			if(game.gameOver) return;
			var secs:String = int(game.gameTimer).toString();
			if(secs.length == 1) secs = "0" + secs;
			var mils:String = int((game.gameTimer*100)%100).toString();
			if(mils.length == 1) mils = "0" + mils;
			timer1.text = timer2.text = secs + ":" + mils;
			
			var scale:Number = game.gameTimer / 60;
			quad1.scaleX = quad2.scaleX = quad3.scaleX = quad4.scaleX = scale;
		}
	}
}