package Game
{
	import starling.display.Image;
	import starling.display.Sprite;
	import starling.filters.ColorMatrixFilter;
	import starling.textures.Texture;
	
	public class Hero extends Sprite
	{
		public var game:GameScene;
		public var textures:Vector.<Texture>;
		public var image:Image;
		public var active:Boolean;
		
		public var turnRate:Number;
		public var maxTurnRate:Number
		public var speed:Number;
		public var maxSpeed:Number;
		public var vector:NumPoint;
		
		public var reload:Number;
		public var fireAlt:int;
		
		public var lastTrail:TrailFX;
		public var lastTrail2:TrailFX;
		public var trailTimer:Number;
		
		public function Hero(game:GameScene)
		{
			this.game = game;
			
			turnRate = 0.1;
			maxTurnRate = 0.2;
			speed = 0;
			maxSpeed = 30;
			vector = new NumPoint(0, 0);
			
			textures = Root.assets.getTextures("lightjet");
			image = new Image(textures[0]);
			image.pivotX = image.width*0.5;
			image.pivotY = image.height*0.5;
			image.color = 0x00aaff;
			image.blendMode = "add";
			addChild(image);
		}
		
		public function init():void {
			game.shipLayer.addChild(this);
			x = 400;
			y = 300;
			lastTrail = null;
			lastTrail2 = null;
			trailTimer = 0;
			reload = 0;
			fireAlt = 0;
			image.rotation = -Math.PI*0.5;
			updateFacing();
			active = true;
		}
		
		public function deInit():void {
			lastTrail = null;
			game.shipLayer.removeChild(this);
		}
		
		public function update(dt:Number):void {
			//movement
			if(game.ui.mousePresent) {
				var dx:Number = game.ui.mouseLoc.x - 400;
				var dy:Number = game.ui.mouseLoc.y - 300;
				var angle:Number = Math.atan2(dy, dx);
				var dist:Number = Math.sqrt(dx*dx + dy*dy);
			} else {
				angle = image.rotation;
				dist = 200;
			}
			
			
			speed = dist*0.15;
			if(speed > maxSpeed) speed = maxSpeed;
			turnRate = 0.1 - ((speed/maxSpeed)*0.07);
			
			var diff:Number = angle - image.rotation;
			var angleDiff:Number = Math.atan2(Math.sin(diff), Math.cos(diff));
			if(angleDiff > 0) {
				if(turnRate < angleDiff) image.rotation += turnRate;
				else image.rotation += angleDiff;
			} else {
				if(-turnRate > angleDiff) image.rotation -= turnRate;
				else image.rotation += angleDiff;
			}
			
			vector.x += Math.cos(image.rotation)*speed*dt;
			vector.y += Math.sin(image.rotation)*speed*dt;
			
			vector.x *= 0.96;
			vector.y *= 0.96;
			
			this.x += vector.x;
			this.y += vector.y;
			
			updateFacing();
			
			//wingtip trails
			
			var halfPi:Number = Math.PI*0.5;
			var wingDist:Number = 24;
			var ratio:Number = (Math.abs(image.rotation)-halfPi) / halfPi;
			ratio = 1 - Math.abs(ratio);
			wingDist *= ratio;
			var yOffset:Number = 6 * (1-ratio);
			
			if(lastTrail) {
				dx = (this.x-Math.cos(image.rotation+halfPi)*wingDist) - lastTrail.x;
				dy = (this.y-Math.sin(image.rotation+halfPi)*wingDist) - lastTrail.y + yOffset;
				dist = Math.sqrt(dx*dx + dy*dy);
				lastTrail.rotation = 0;
				lastTrail.width = dist;
				lastTrail.rotation = Math.atan2(dy, dx);
			}
			
			if(lastTrail2) {
				dx = (this.x-Math.cos(image.rotation-halfPi)*wingDist) - lastTrail2.x;
				dy = (this.y-Math.sin(image.rotation-halfPi)*wingDist) - lastTrail2.y + yOffset;
				dist = Math.sqrt(dx*dx + dy*dy);
				lastTrail2.rotation = 0;
				lastTrail2.width = dist;
				lastTrail2.rotation = Math.atan2(dy, dx);
			}
			
			trailTimer -= dt;
			if(trailTimer <= 0) {
				lastTrail = game.addTrail((this.x-Math.cos(image.rotation+halfPi)*wingDist), (this.y-Math.sin(image.rotation+halfPi)*wingDist) + yOffset, image.rotation, 0, 0x006699);
				lastTrail2 = game.addTrail((this.x-Math.cos(image.rotation-halfPi)*wingDist), (this.y-Math.sin(image.rotation-halfPi)*wingDist) + yOffset, image.rotation, 0, 0x006699);
				trailTimer = 0.06;
			}
			
			//shooting
			if(reload > 0) {
				reload -= dt;
				if(reload < 0) reload = 0;
			}
			
			var fireDist:Number = 12*ratio;
			if(reload == 0 && game.ui.mouseDown) {
				reload = 0.1;
				if(fireAlt == 0){
					var xpos:Number = x-Math.cos(image.rotation+halfPi)*fireDist;
					var ypos:Number = y-Math.sin(image.rotation+halfPi)*fireDist;
					game.addProjectile(xpos, ypos, vector, image.rotation, 0, image.color);
					fireAlt = 1;
				} else {
					xpos = x-Math.cos(image.rotation-halfPi)*fireDist;
					ypos = y-Math.sin(image.rotation-halfPi)*fireDist;
					game.addProjectile(xpos, ypos, vector, image.rotation, 0, image.color);
					fireAlt = 0;
				}
			}
		}
		
		public function updateFacing():void {
			var facing:int;
			var steps:Number = (Math.PI*2) / 128;
			if(image.rotation > 0) {
				facing = 64 + Math.round(image.rotation/steps);
				image.scaleY = -1;
			} else {
				facing = Math.round(-image.rotation/steps);
				image.scaleY = 1;
			}
			image.texture = textures[facing];
		}
		
		public function hit():void {
			game.minusTime();
			GS.playHit();
		}
	}
}