package Game
{
	import starling.display.Image;

	public class ParticleFX
	{
		public var game:GameScene;
		public var active:Boolean;
		public var image:Image;
		public var maxScale:Number;
		
		public var type:int;
		public static const TYPE_HIT:int = 0;
		public static const TYPE_SHOCK:int = 1;
		public static const TYPE_FLARE:int = 2;
		public static const TYPE_SMOKE:int = 3;
		public static const TYPE_SHARD:int = 4; 
		
		public var state:int;
		public static const STATE_BLOOM:int = 0;
		public static const STATE_WILT:int = 1;
		
		public function ParticleFX(game:GameScene)
		{
			this.game = game;
			
			image = new Image(Root.assets.getTexture("circle"));
			image.blendMode = "add";
		}
		
		public function init(x:Number, y:Number):void {
			
		}
		
		public function initHit(x:Number, y:Number, color:uint):void {
			image.texture = Root.assets.getTexture("circle");
			image.readjustSize();
			image.pivotX = image.texture.width*0.5;
			image.pivotY = image.texture.height*0.5;
			
			image.x = x;
			image.y = y;
			image.alpha = 1;
			image.color = color;
			image.scaleX = image.scaleY = 0.6;
			type = TYPE_HIT;
			state = STATE_WILT;
			game.particleLayer.addChild(image);
			active = true;
		}
		
		public function initShock(x:Number, y:Number, color:uint):void {
			image.texture = Root.assets.getTexture("halo");
			image.readjustSize();
			image.pivotX = image.texture.width*0.5;
			image.pivotY = image.texture.height*0.5;
			
			image.x = x;
			image.y = y;
			image.alpha = 1;
			image.color = 0xffaa00;
			image.scaleX = image.scaleY = 0;
			type = TYPE_SHOCK;
			state = STATE_BLOOM;
			game.particleLayer.addChild(image);
			active = true;
		}
		
		public function initSmoke(x:Number, y:Number, color:uint):void {
			image.texture = Root.assets.getTexture("circle2");
			image.readjustSize();
			image.pivotX = image.texture.width*0.5;
			image.pivotY = image.texture.height*0.5;
			
			image.x = x;
			image.y = y;
			image.alpha = 1;
			image.color = color;
			image.scaleX = image.scaleY = Math.random()*0.3 + 0.3;
			type = TYPE_SMOKE;
			state = STATE_WILT;
			game.particleLayer.addChild(image);
			active = true;
		}
		
		public var vx:Number;
		public var vy:Number;
		public function initShard(x:Number, y:Number, vector:NumPoint, color:uint):void {
			image.texture = Root.assets.getTexture("bullet");
			image.readjustSize();
			image.pivotX = image.texture.width*0.5;
			image.pivotY = image.texture.height*0.5;
			
			vx = vector.x;
			vy = vector.y;
			image.x = x;
			image.y = y;
			image.alpha = 1;
			image.color = 0xffaa00;
			image.scaleX = image.scaleY = 2;
			image.rotation = Math.PI*Math.random();
			if(Math.random() < 0.5) image.rotation *= -1;
			type = TYPE_SHARD;
			game.particleLayer.addChild(image);
			active = true;
		}
		
		public function initFlare(x:Number, y:Number, color:uint):void {
			image.texture = Root.assets.getTexture("circle2");
			image.readjustSize();
			image.pivotX = image.texture.width*0.5;
			image.pivotY = image.texture.height*0.5;
			
			image.x = x;
			image.y = y;
			image.alpha = 1;
			image.color = 0xffaa00;
			image.scaleX = image.scaleY = Math.random();
			type = TYPE_FLARE;
			state = STATE_BLOOM;
			game.particleLayer.addChild(image);
			active = true;
			maxScale = 0.7;
		}
		
		public function initFlare2(x:Number, y:Number, color:uint):void {
			image.texture = Root.assets.getTexture("circle2");
			image.readjustSize();
			image.pivotX = image.texture.width*0.5;
			image.pivotY = image.texture.height*0.5;
			
			image.x = x;
			image.y = y;
			image.alpha = 1;
			image.color = 0xffaa00;
			image.scaleX = image.scaleY = Math.random()*1.5;
			type = TYPE_FLARE;
			state = STATE_BLOOM;
			game.particleLayer.addChild(image);
			active = true;
			maxScale = 2;
		}
		
		public function deInit():void {
			game.particleLayer.removeChild(image);
		}
		
		public function update(dt:Number):void {
			switch(type) {
				case TYPE_HIT:
					image.scaleX -= dt*2;
					image.scaleY -= dt*2;
					image.alpha = 0.5 + Math.random()*0.5;
					if(image.scaleX <= 0) {
						active = false;
					}
					break;
				case TYPE_SHOCK:
					if(state == STATE_BLOOM) {
						image.scaleX += dt*25;
						image.scaleY += dt*25;
						image.alpha -= dt*5;
						if(image.alpha <= 0) {
							active = false;
						}
					}
					break;
				case TYPE_FLARE:
					if(state == STATE_BLOOM) {
						image.scaleX += dt*20;
						image.scaleY += dt*20;
						if(image.scaleX > maxScale) {
							image.scaleX = image.scaleY = maxScale;
							state = STATE_WILT;
						}
					} else {
						if(maxScale == 0.7) {
							image.scaleX -= dt;
							image.scaleY -= dt;
						} else {
							image.scaleX -= dt*2;
							image.scaleY -= dt*2;
						}
						image.alpha = 0.3 + Math.random()*0.3;
						if(image.scaleX <= 0) {
							active = false;
						}
					}
					break;
				case TYPE_SMOKE:
					image.scaleX -= dt*0.5;
					image.scaleY -= dt*0.5;
					image.alpha = 0.5 + Math.random()*0.5;
					if(image.scaleX <= 0) {
						active = false;
					}
					break;
				case TYPE_SHARD:
					image.x += vx + Math.cos(image.rotation)*1000*dt;
					image.y += vy + Math.sin(image.rotation)*1000*dt;
					image.alpha = 0.5 + Math.random()*0.5;
					image.scaleX -= dt*10;
					image.scaleY -= dt*10;
					if(image.scaleX <= 0) {
						active = false;
					}
					break;
			}
		}
	}
}