package Game
{
	import starling.core.Starling;
	import starling.display.Image;
	import starling.display.Sprite;
	import starling.filters.ColorMatrixFilter;
	import starling.textures.Texture;
	
	public class Ship extends Sprite
	{
		public var game:GameScene;
		public var textures:Vector.<Texture>;
		public var image:Image;
		public var active:Boolean;
		
		public var turnRate:Number;
		public var maxTurnRate:Number
		public var speed:Number;
		public var maxSpeed:Number;
		public var vector:NumPoint;
		public var cursor:NumPoint;
		
		public var reload:Number;
		public var fireAlt:int;
		public var turnTimer:Number;
		public var turnReload:Number;
		
		public var lastTrail:TrailFX;
		public var lastTrail2:TrailFX;
		public var trailTimer:Number;
		
		public var hp:int;
		public var smokeTimer:Number;
		public var dying:Boolean;
		public var deathTimer:Number;
		public var hasPowerUp:Boolean;
		
		public function Ship(game:GameScene)
		{
			this.game = game;
			
			turnRate = 0.1;
			maxTurnRate = 0.2;
			speed = 0;
			maxSpeed = 20;
			vector = new NumPoint(0, 0);
			cursor = new NumPoint(0, 0);
			
			textures = Root.assets.getTextures("lightjet");
			image = new Image(textures[0]);
			image.pivotX = image.width*0.5;
			image.pivotY = image.height*0.5;
			image.color = 0xff0000;
			image.blendMode = "add";
			addChild(image);
		}
		
		public function init(x:Number, y:Number):void {
			game.shipLayer.addChild(this);
			this.x = x;
			this.y = y;
			lastTrail = null;
			trailTimer = 0;
			turnReload = 1+Math.random()*2;
			turnTimer = 1+Math.random()*3;
			reload = 0.5;
			fireAlt = 0;
			cursor.x = 0;
			cursor.y = 0;
			vector.x = 0;
			vector.y = 0;
			smokeTimer = 0;
			hp = 100;
			active = true;
			dying = false;
			hasPowerUp = false;
			deathTimer = 1 + Math.random();
			
			image.rotation = Math.random()*Math.PI;
			if(Math.random() < 0.5) image.rotation *= -1;
			updateFacing();
		}
		
		public function deInit():void {
			lastTrail = null;
			lastTrail2 = null;
			game.shipLayer.removeChild(this);
		}
		
		public function update(dt:Number):void {
			if(dying) {
				updateDeath(dt);
				return;
			}
			
			//targeting
			cursor.x = game.hero.x;
			cursor.y = game.hero.y;
			
			//movement
			var dx:Number = cursor.x - this.x;
			var dy:Number = cursor.y - this.y;
			var angle:Number = Math.atan2(dy, dx);
			var dist:Number = Math.sqrt(dx*dx + dy*dy);
			var targetDist:Number = dist;
			
			//speed = dist*0.15;
			speed = 20;
			if(speed > maxSpeed) speed = maxSpeed;
			turnRate = 0.02;
			
			var diff:Number = angle - image.rotation;
			var angleDiff:Number = Math.atan2(Math.sin(diff), Math.cos(diff));
			
			if(turnReload > 0) {
				turnReload -= dt;
			}
			if(turnReload <= 0 && turnTimer > 0) {
				turnTimer -= dt;
				if(angleDiff > 0) {
					if(turnRate < angleDiff) image.rotation += turnRate;
					else image.rotation += angleDiff;
				} else {
					if(-turnRate > angleDiff) image.rotation -= turnRate;
					else image.rotation += angleDiff;
				}
			}
			if(turnTimer <= 0 && turnReload <= 0) {
				turnReload = 1+Math.random()*2;
				turnTimer = 1+Math.random()*3;
			}
			
			vector.x += Math.cos(image.rotation)*speed*dt;
			vector.y += Math.sin(image.rotation)*speed*dt;
			
			vector.x *= 0.96;
			vector.y *= 0.96;
			
			this.x += vector.x;
			this.y += vector.y;
			
			updateFacing();
			updateTrails(dt);
			
			//shooting
			if(reload > 0) {
				reload -= dt;
				if(reload < 0) reload = 0;
			}
			
			
			if(reload == 0 && targetDist < 1000 && Math.abs(angleDiff) < 0.05) {
				reload = 0.1;
				var halfPi:Number = Math.PI*0.5;
				var ratio:Number = (Math.abs(image.rotation)-halfPi) / halfPi;
				ratio = 1 - Math.abs(ratio);
				var fireDist:Number = 12*ratio;
				if(fireAlt == 0){
					game.addProjectile(x-Math.cos(image.rotation+halfPi)*fireDist, y-Math.sin(image.rotation+halfPi)*fireDist, vector, image.rotation, 1, image.color);
					fireAlt = 1;
				} else {
					game.addProjectile(x-Math.cos(image.rotation-halfPi)*fireDist, y-Math.sin(image.rotation-halfPi)*fireDist, vector, image.rotation, 1, image.color);
					fireAlt = 0;
				}
			}
			
			//smoke trail
			if(hp <= 60) {
				smokeTimer -= dt;
				if(smokeTimer <= 0) {
					game.addSmoke(x+Math.random()*40-20, y+Math.random()*40-20, image.color);
					smokeTimer = 0.1 + (hp/60)*0.3;
				}
			}
		}
		
		public function updateDeath(dt:Number):void {
			vector.x += Math.cos(image.rotation)*speed*dt;
			vector.y += Math.sin(image.rotation)*speed*dt;
			vector.x *= 0.96;
			vector.y *= 0.96;
			this.x += vector.x;
			this.y += vector.y;
			
			facing += 3;
			if(facing > 127) facing = 0;
			image.texture = textures[facing];
			
			//trails
			var dx:Number, dy:Number, dist:Number;
			var halfPi:Number = Math.PI*0.5;
			var wingDist:Number = 24;
			var ratio:Number = (facing % 64) / 32;
			if(ratio > 1) ratio = 1 - (ratio-1);
			wingDist *= ratio;
			var yOffset:Number = 6 * (1-ratio);
			
			if(lastTrail) {
				dx = (this.x-Math.cos(image.rotation+halfPi)*wingDist) - lastTrail.x;
				dy = (this.y-Math.sin(image.rotation+halfPi)*wingDist) - lastTrail.y + yOffset;
				dist = Math.sqrt(dx*dx + dy*dy);
				lastTrail.rotation = 0;
				lastTrail.width = dist;
				lastTrail.rotation = Math.atan2(dy, dx);
			}
			
			if(lastTrail2) {
				dx = (this.x-Math.cos(image.rotation-halfPi)*wingDist) - lastTrail2.x;
				dy = (this.y-Math.sin(image.rotation-halfPi)*wingDist) - lastTrail2.y + yOffset;
				dist = Math.sqrt(dx*dx + dy*dy);
				lastTrail2.rotation = 0;
				lastTrail2.width = dist;
				lastTrail2.rotation = Math.atan2(dy, dx);
			}
			
			trailTimer -= dt;
			if(trailTimer <= 0) {
				lastTrail = game.addTrail((this.x-Math.cos(image.rotation+halfPi)*wingDist), (this.y-Math.sin(image.rotation+halfPi)*wingDist) + yOffset, image.rotation, 0, 0xaa0000);
				lastTrail2 = game.addTrail((this.x-Math.cos(image.rotation-halfPi)*wingDist), (this.y-Math.sin(image.rotation-halfPi)*wingDist) + yOffset, image.rotation, 0, 0xaa0000);
				trailTimer = 0.06;
			}
			
			//smoke
			smokeTimer -= dt;
			if(smokeTimer <= 0) {
				game.addSmoke(x+Math.random()*40-20, y+Math.random()*40-20, image.color);
				game.addFlare(x+Math.random()*40-20, y+Math.random()*40-20, image.color);
				smokeTimer = 0.1;
			}
			
			deathTimer -= dt;
			if(deathTimer <= 0) {
				active = false;
				game.addShockParticle(x, y, image.color);
				for(var i:int = 0; i < 10; i++) game.addShard(x, y, vector, image.color);
				for(i = 0; i < 6; i++) game.addFlare2(x+Math.random()*60-30, y+Math.random()*60-30, image.color);
				if(hasPowerUp) game.addPowerup(x, y, vector);
				if(game.ships.length == 1) Starling.juggler.delayCall(game.checkNextWave, 1.5);
				game.kills++;
				GS.playExplosion();
			}
		}
		
		public var facing:int;
		public function updateFacing():void {
			var steps:Number = (Math.PI*2) / 128;
			if(image.rotation > 0) {
				facing = 64 + Math.round(image.rotation/steps);
				image.scaleY = -1;
			} else {
				facing = Math.round(-image.rotation/steps);
				image.scaleY = 1;
			}
			image.texture = textures[facing];
		}
		
		public function updateTrails(dt:Number):void {
			var dx:Number, dy:Number, dist:Number;
			var halfPi:Number = Math.PI*0.5;
			var wingDist:Number = 24;
			var ratio:Number = (Math.abs(image.rotation)-halfPi) / halfPi;
			ratio = 1 - Math.abs(ratio);
			wingDist *= ratio;
			var yOffset:Number = 6 * (1-ratio);
			
			if(lastTrail) {
				dx = (this.x-Math.cos(image.rotation+halfPi)*wingDist) - lastTrail.x;
				dy = (this.y-Math.sin(image.rotation+halfPi)*wingDist) - lastTrail.y + yOffset;
				dist = Math.sqrt(dx*dx + dy*dy);
				lastTrail.rotation = 0;
				lastTrail.width = dist;
				lastTrail.rotation = Math.atan2(dy, dx);
			}
			
			if(lastTrail2) {
				dx = (this.x-Math.cos(image.rotation-halfPi)*wingDist) - lastTrail2.x;
				dy = (this.y-Math.sin(image.rotation-halfPi)*wingDist) - lastTrail2.y + yOffset;
				dist = Math.sqrt(dx*dx + dy*dy);
				lastTrail2.rotation = 0;
				lastTrail2.width = dist;
				lastTrail2.rotation = Math.atan2(dy, dx);
			}
			
			trailTimer -= dt;
			if(trailTimer <= 0) {
				lastTrail = game.addTrail((this.x-Math.cos(image.rotation+halfPi)*wingDist), (this.y-Math.sin(image.rotation+halfPi)*wingDist) + yOffset, image.rotation, 0, 0xaa0000);
				lastTrail2 = game.addTrail((this.x-Math.cos(image.rotation-halfPi)*wingDist), (this.y-Math.sin(image.rotation-halfPi)*wingDist) + yOffset, image.rotation, 0, 0xaa0000);
				trailTimer = 0.06;
			}
		}
		
		public function hit():void {
			GS.playHit();
			if(dying || !active) return;
			hp -= 10;
			if(hp <= 0) {
				hp = 0;
				dying = true;
				deathTimer = 1.5 + Math.random()*1;
				game.addShockParticle(x, y, image.color);
				for(var i:int = 0; i < 8; i++) game.addShard(x, y, vector, image.color);
			}
		}
	}
}